@@ -2,7 +2,7 @@
 """
 A test harness for gnupg.py.
 
-Copyright (C) 2008-2017 Vinay Sajip. All rights reserved.
+Copyright (C) 2008-2018 Vinay Sajip. All rights reserved.
 """
 import doctest
 import logging
@@ -28,7 +28,7 @@ except ImportError:
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$06-Jul-2017 15:09:39$"
+__date__  = "$28-Mar-2018 15:27:14$"
 
 ALL_TESTS = True
 
@@ -321,7 +321,13 @@ class GPGTestCase(unittest.TestCase):
         fp = key_info['fingerprint']
         self.assertTrue(fp in public_keys.key_map)
         self.assertTrue(public_keys.key_map[fp] is key_info)
-        for _, _, sfp in key_info['subkeys']:
+        self.assertTrue('subkey_info' in key_info)
+        skinfo = key_info['subkey_info']
+        for skid, _, sfp in key_info['subkeys']:
+            self.assertTrue(skid in skinfo)
+            info = skinfo[skid]
+            self.assertEqual(info['keyid'], skid)
+            self.assertEqual(info['type'], 'sub')
             self.assertTrue(sfp in public_keys.key_map)
             self.assertTrue(public_keys.key_map[sfp] is key_info)
 
@@ -334,9 +340,15 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(fp in public_keys_sigs.key_map)
         self.assertTrue(public_keys_sigs.key_map[fp] is key_info)
         self.assertTrue(is_list_with_len(key_info['sigs'], 2))
+        self.assertTrue('subkey_info' in key_info)
+        skinfo = key_info['subkey_info']
         for siginfo in key_info['sigs']:
             self.assertTrue(len(siginfo), 3)
-        for _, _, sfp in key_info['subkeys']:
+        for skid, _, sfp in key_info['subkeys']:
+            self.assertTrue(skid in skinfo)
+            info = skinfo[skid]
+            self.assertEqual(info['keyid'], skid)
+            self.assertEqual(info['type'], 'sub')
             self.assertTrue(sfp in public_keys_sigs.key_map)
             self.assertTrue(public_keys_sigs.key_map[sfp] is key_info)
 
@@ -344,6 +356,14 @@ class GPGTestCase(unittest.TestCase):
         self.assertTrue(is_list_with_len(private_keys, 1),
                         "1-element list expected")
         self.assertEqual(len(private_keys.fingerprints), 1)
+        key_info = private_keys[0]
+        self.assertTrue('subkey_info' in key_info)
+        skinfo = key_info['subkey_info']
+        self.assertTrue(skid in skinfo)
+        info = skinfo[skid]
+        self.assertEqual(info['keyid'], skid)
+        self.assertEqual(info['type'], 'ssb')
+
         # Now do the same test, but using keyring and secret_keyring arguments
         if self.gpg.version < (2, 1):
             pkn = 'pubring.gpg'
@@ -399,6 +419,29 @@ class GPGTestCase(unittest.TestCase):
                         'Donna Davis <donna.davis@delta.com>'])
         self.assertEqual(actual, expected)
 
+    def test_key_trust(self):
+        "Test that trusting keys works"
+        gpg = self.gpg
+        gpg.import_keys(KEYS_TO_IMPORT)
+        keys = gpg.list_keys()
+        fingerprints = []
+        for key in keys:
+            self.assertEqual(key['ownertrust'], '-')
+            fingerprints.append(key['fingerprint'])
+        cases = (
+            ('TRUST_NEVER', 'n'),
+            ('TRUST_MARGINAL', 'm'),
+            ('TRUST_FULLY', 'f'),
+            ('TRUST_ULTIMATE', 'u'),
+            ('TRUST_UNDEFINED', 'q'),
+        )
+        for param, expected in cases:
+            gpg.trust_keys(fingerprints, param)
+            keys = gpg.list_keys(keys=fingerprints)
+            for key in keys:
+                self.assertEqual(key['ownertrust'], expected)
+        self.assertRaises(ValueError, gpg.trust_keys, fingerprints, 'TRUST_FOOBAR')
+
     def test_list_signatures(self):
         logger.debug("test_list_signatures begins")
         imported = self.gpg.import_keys(SIGNED_KEYS)
@@ -564,8 +607,14 @@ class GPGTestCase(unittest.TestCase):
         self.assertFalse(sig, "Bad passphrase should fail")
         sig = self.gpg.sign(data, keyid=key.fingerprint, passphrase='aable')
         self.assertTrue(sig, "Good passphrase should succeed")
+        if sig.username:  # not set in recent versions of GnuPG e.g. 2.2.5
+            self.assertTrue(sig.username.startswith('Andrew Able'))
+        if sig.key_id:
+            self.assertTrue(key.fingerprint.endswith(sig.key_id))
         self.assertTrue(sig.hash_algo)
+        logger.debug('verification start')
         verified = self.gpg.verify(sig.data)
+        logger.debug('verification end')
         if key.fingerprint != verified.fingerprint:  # pragma: no cover
             logger.debug("key: %r", key.fingerprint)
             logger.debug("ver: %r", verified.fingerprint)
@@ -646,6 +695,8 @@ class GPGTestCase(unittest.TestCase):
         data_file.close()
         try:
             verified = self.gpg.verify_data(sig_file, data)
+            self.assertTrue(verified.username.startswith('Andrew Able'))
+            self.assertTrue(key.fingerprint.endswith(verified.key_id))
         finally:
             os.unlink(sig_file)
         if key.fingerprint != verified.fingerprint:
@@ -670,8 +721,9 @@ class GPGTestCase(unittest.TestCase):
 
     def test_nogpg(self):
         "Test that absence of gpg is handled correctly"
-        self.assertRaises(OSError, gnupg.GPG, gnupghome=self.homedir,
-                          gpgbinary='frob')
+        with self.assertRaises(OSError) as ar:
+            gnupg.GPG(gnupghome=self.homedir, gpgbinary='frob')
+        self.assertIn('frob', str(ar.exception))
 
     def test_make_args(self):
         "Test argument line construction"
@@ -748,6 +800,27 @@ class GPGTestCase(unittest.TestCase):
         self.do_file_encryption_and_decryption(encfname, decfname)
         logger.debug("test_file_encryption_and_decryption ends")
 
+    #@skipIf(os.name == 'nt', 'Test not suitable for Windows')
+    def test_invalid_outputs(self):
+        "Test encrypting to invalid output files"
+        encfno, encfname = tempfile.mkstemp()
+        os.close(encfno)
+        cases = (
+            ('/dev/null/foo', 'not a directory'),
+            ('/etc/foo', 'permission denied'),
+        )
+        key = self.generate_key("Barbara", "Brown", "beta.com")
+        barbara = key.fingerprint
+        data = "Hello, world!"
+        for badout, message in cases:
+            stream = gnupg._make_binary_stream(data, self.gpg.encoding)
+            edata = self.gpg.encrypt_file(stream,
+                                          barbara, armor=False, output=badout)
+            # on GnuPG 1.4, you sometimes don't get any FAILURE messages, in
+            # which case status will not be set
+            if edata.status:
+                self.assertEqual(edata.status, message)
+
     def test_filenames_with_spaces(self):       # See Issue #16
         "Test that filenames with spaces are correctly handled"
         logger.debug("test_filename_with_spaces begins")
@@ -899,10 +972,11 @@ class GPGTestCase(unittest.TestCase):
 
 
 TEST_GROUPS = {
-    'sign' : set(['test_signature_verification']),
+    'sign' : set(['test_signature_verification',
+                  'test_signature_file']),
     'crypt' : set(['test_encryption_and_decryption',
                    'test_file_encryption_and_decryption',
-                   'test_filenames_with_spaces']),
+                   'test_filenames_with_spaces', 'test_invalid_outputs']),
     'key' : set(['test_deletion', 'test_import_and_export',
                  'test_list_keys_after_generation',
                  'test_list_signatures',
@@ -910,12 +984,12 @@ TEST_GROUPS = {
                  'test_key_generation_with_escapes',
                  'test_key_generation_with_empty_value',
                  'test_key_generation_with_colons',
-                 'test_search_keys', 'test_scan_keys']),
+                 'test_search_keys', 'test_scan_keys', 'test_key_trust']),
     'import' : set(['test_import_only', 'test_doctest_import_keys']),
     'basic' : set(['test_environment', 'test_list_keys_initial',
                    'test_nogpg', 'test_make_args',
                    'test_quote_with_shell']),
-    'test': set(['test_doctest_import_keys']),
+    'test': set(['test_signature_verification']),
 }
 
 def suite(args=None):
